unit umain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, ieview, ImageEnView, StdCtrls, ImageEnIO, ComCtrls,
  ExtCtrls, hyieutils, iexBitmaps, hyiedefs, iesettings, iexLayers, 
  iexRulers, iexToolbars, iexFolderTree;

type
  TForm1 = class(TForm)
    Panel2: TPanel;
    ImageEnView1: TImageEnView;
    ProgressBar1: TProgressBar;
    Panel1: TPanel;
    StringGrid1: TStringGrid;
    Panel3: TPanel;
    btnSaveChanges: TButton;
    btnClearIPTC: TButton;
    cmbIPTC: TComboBox;
    IEFolderTree1: TIEFolderTree;
    IEFileListBox1: TIEFileListBox;

    procedure FormCreate(Sender: TObject);
    procedure ImageEnView1Progress(Sender: TObject; per: Integer);
    procedure btnSaveChangesClick(Sender: TObject);
    procedure btnClearIPTCClick(Sender: TObject);
    procedure cmbIPTCChange(Sender: TObject);
    procedure IEFileListBox1SelectionChange(Sender: TObject);
    procedure ImageEnView1FinishWork(Sender: TObject);
  private
    { Private declarations }
    procedure ClearGrid();
    procedure UpdateGrid();
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
{$R ImageEnManifest.res}

uses
  iexMetaHelpers;

const
  // Items of cmbIPTC
  __cmbIPTC_Photoshop_Fields = 0;
  __cmbIPTC_Raw_Fields       = 1;


// Initialize properties grid
procedure TForm1.FormCreate(Sender: TObject);
begin
  cmbIPTC.ItemIndex := __cmbIPTC_Photoshop_Fields;
  StringGrid1.NewGridForIPTC;
  UpdateGrid();
end;

procedure TForm1.ClearGrid();
begin
  if cmbIPTC.ItemIndex = __cmbIPTC_Photoshop_Fields then
    // Use our meta-helper clear method
    StringGrid1.ClearGridFields()
  else
  begin
    StringGrid1.FixedRows := 1;
    StringGrid1.RowCount := StringGrid1.FixedRows + 1;
    StringGrid1.Cells[0, 0] := 'Index';
    StringGrid1.Cells[1, 0] := 'Value';
    StringGrid1.Cells[0, 1] := '';
    StringGrid1.Cells[1, 1] := '';
  end;
end;

procedure TForm1.UpdateGrid();
var
  i: Integer;
  iptcFile, canEdit: Boolean;
  recNo, dataSet: Integer;
  value: string;
begin
  ClearGrid();
  iptcFile := IPTCCompatibleFile( IEFileListBox1.FileName ); // JPEG or TIFF

  canEdit := iptcFile and ( cmbIPTC.ItemIndex = __cmbIPTC_Photoshop_Fields );
  if canEdit then
    StringGrid1.Options := StringGrid1.Options + [goEditing]
  else
    StringGrid1.Options := StringGrid1.Options - [goEditing];
  btnClearIPTC   .Enabled := canEdit;
  btnSaveChanges .Enabled := canEdit;

  if NOT iptcFile then
    exit;

  if cmbIPTC.ItemIndex = __cmbIPTC_Photoshop_Fields then
  begin
    // Use our helper to load Photoshop IPTC fields
    StringGrid1.NewGridForIPTC;
    StringGrid1.ReadGridFromIPTC(ImageEnView1.IO.Params);
  end
  else
  begin
    // read raw IPTC fields
    for i := 0 to ImageEnView1.IO.Params.IPTC_Info.Count - 1 do
    begin
      if i > 0 then
        StringGrid1.RowCount := StringGrid1.RowCount + 1;

      recNo   := ImageEnView1.IO.Params.IPTC_Info.RecordNumber[i];
      dataSet := ImageEnView1.IO.Params.IPTC_Info.DataSet[i];
      value   := ImageEnView1.IO.Params.IPTC_Info.StringItem[i];

      StringGrid1.Cells[0, StringGrid1.FixedRows + i] := format( '(%d, %d)', [ recNo, dataSet ]);
      StringGrid1.Cells[1, StringGrid1.FixedRows + i] := TidyIPTCStr(value);
    end;
  end;
end;

// Show progress
procedure TForm1.ImageEnView1Progress(Sender: TObject; per: Integer);
begin
  ProgressBar1.Position := per;
end;

// Reset progress
procedure TForm1.ImageEnView1FinishWork(Sender: TObject);
begin
  ProgressBar1.Position := 0;
end;

// Save changes
procedure TForm1.btnSaveChangesClick(Sender: TObject);
var
  bIPTCChanged: Boolean;
begin
  if cmbIPTC.ItemIndex <> __cmbIPTC_Photoshop_Fields then
    exit;

  bIPTCChanged := StringGrid1.WriteGridToIPTC( ImageEnView1.IO.Params );

  if bIPTCChanged then
    case ImageEnView1.io.Params.FileType of
      ioJPEG:
        // inject iptc in jpeg
        ImageEnView1.io.InjectJpegIPTC(IEFileListBox1.FileName);
      ioTIFF:
        // save iptc in TIFF
        ImageEnView1.io.SaveToFile(IEFileListBox1.FileName);
    end;
end;

// Clear grid
procedure TForm1.btnClearIPTCClick(Sender: TObject);
begin
  ImageEnView1.io.Params.IPTC_Info.Clear;
  ClearGrid();
end;

procedure TForm1.cmbIPTCChange(Sender: TObject);
begin
  UpdateGrid();
end;

// File selected
procedure TForm1.IEFileListBox1SelectionChange(Sender: TObject);
begin
  if IsKnownFormat( IEFileListBox1.FileName) then
  begin
    // we need only a thumbnail (fast load)
    ImageEnView1.io.Params.Width := ImageEnView1.Width;
    ImageEnView1.io.Params.Height := ImageEnView1.Height;
    ImageEnView1.io.Params.JPEG_Scale := ioJPEG_AUTOCALC;
    ImageEnView1.io.LoadFromFile(IEFileListBox1.FileName);
  end;

  UpdateGrid();
end;

end.
